==> wxPostgres-0.2.1.tar.gz: wxPostgres/wxPg/db/cached_result.h <==
/* cached_result.h - class wxPg::Cached_result for storing locally entire
    result of SQL query.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __CACHED_RESULT_H
#define __CACHED_RESULT_H

#include "wxPg/conf.h"
#include "wxPg/db/result_data_provider.h"
#include<vector>
#include<map>

namespace wxPg{

//foreward declaration
class Database;

/** Stores entire result of query locally.
    Don't use it for storing very large results.
*/
class wxPG_DECLSPEC
Cached_result : public Result_data_provider{
public:
    /** Create new empty object. */
    Cached_result();

    /** Copy constructor, creates a copy of other Cached_result.
        Note, that source object will be entirely copied,
        no memory will be shared between two objects.
        @param src - a source object to be copied.
    */
    Cached_result(const Cached_result &src);

    /** Assign one Cached result to other.
        Any data, contained by this object will be freed.\n
        Note, that source object will be entirely copied,
        no memory will be shared between two objects.
        @param src - a source object to be copied.
    */
    Cached_result&
    operator=(const Cached_result &src);

    /** Destroy this object, fee resources. */
    virtual
    ~Cached_result();

    /** Store PGresult to this object.
        Used internally. Cleans this object and stores PGresult.
    */
    void
    store(const Database &db, PGresult *res);

    /** Clear contents of this object, fee resources. */
    void
    clear();

    //inherited
    virtual int
    cols();

    //inherited
    virtual const wxString
    col_name(int idx);

    //inherited
    virtual int
    col_idx(const wxString &field);

    //inherited
    virtual int
    type_simple(int idx);

    //inherited
    virtual int
    type_simple(const wxString &field);

    //inherited
    virtual bool
    null(int idx);

    //inherited
    virtual bool
    null(const wxString &field);

    //inherited
    virtual int
    get_int(int idx);

    //inherited
    virtual int
    get_int(const wxString &field);

    //inherited
    virtual double
    get_float(int idx);

    //inherited
    virtual double
    get_float(const wxString &field);

    //inherited
    virtual const wxString
    get_str(int idx);

    //inherited
    virtual const wxString
    get_str(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date(int idx);

    //inherited
    virtual const wxDateTime
    get_date(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_time(int idx);

    //inherited
    virtual const wxDateTime
    get_time(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date_time(int idx);

    //inherited
    virtual const wxDateTime
    get_date_time(const wxString &field);

    //inherited
    virtual bool
    get_bool(int idx);

    //inherited
    virtual bool
    get_bool(const wxString &field);

    //inherited
    virtual int
    rows();

    //inherited
    virtual bool
    current(int idx);

    //inherited
    virtual int
    current()const;
private:
    //map field name to index
    std::map<wxString, unsigned>
    field_indices_;

    //field types
    std::vector<int>
    field_types_;

    //a single value with type info
    struct
    value_info_{
        wxString value;
        bool null;
    };

    //locally stored data
    std::vector<std::vector<value_info_> >
    data_;

    //copy other object
    inline void
    make_copy_(const Cached_result &src);

    //find field_info by index
    inline std::map<wxString, unsigned>::iterator
    find_field_idx_(int idx);

    //check if index is in range
    inline bool
    check_idx_(int idx);

    //check if data exists
    inline bool
    check_data_(int idx);

    //check if data exists
    inline int
    check_data_(const wxString &field);

    //currently active row
    unsigned
    current_;
};

}

#endif

==> wxPostgres-0.3.1.tar.gz: wxPostgres/wxPg/db/cached_result.h <==
/* cached_result.h - class wxPg::Cached_result for storing locally entire
    result of SQL query.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __CACHED_RESULT_H
#define __CACHED_RESULT_H

#include "wxPg/conf.h"
#include "wxPg/db/result_data_provider.h"
#include "wxPg/db/cached_row_provider.h"
#include<vector>
#include<map>


//let use library without libpq-fe.h
#ifndef LIBPQ_FE_H
extern "C"{
typedef struct pg_result PGresult;
}
#endif


namespace wxPg{

/** Stores entire result of query locally.
    Don't use it for storing very large results.
*/
class wxPG_DECLSPEC
Cached_result : virtual public Result_data_provider,
                virtual public Cached_row_provider{
public:
    /** Create new empty object. */
    Cached_result();

    /** Copy constructor, creates a copy of other Cached_result.
        Note, that source object will be entirely copied,
        no memory will be shared between two objects.
        @param src - a source object to be copied.
    */
    Cached_result(const Cached_result &src);

    /** Assign one Cached result to other.
        Any data, contained by this object will be freed.\n
        Note, that source object will be entirely copied,
        no memory will be shared between two objects.
        @param src - a source object to be copied.
    */
    Cached_result&
    operator=(const Cached_result &src);

    /** Destroy this object, fee resources. */
    virtual
    ~Cached_result();

    /** Store PGresult to this object.
        Used internally. Cleans this object and stores PGresult.
    */
    void
    store(const Database &db, PGresult *res);

    /** Clear contents of this object, fee resources. */
    void
    clear();

    //inherited
    virtual int
    cols();

    //inherited
    virtual const wxString
    col_name(int idx);

    //inherited
    virtual int
    col_idx(const wxString &field);

    //inherited
    virtual int
    type_simple(int idx);

    //inherited
    virtual bool
    null(int idx);

    //inherited
    virtual const wxString
    get_str(int idx);

    //inherited
    virtual int
    rows();

    //inherited
    virtual bool
    current(int idx);

    //inherited
    virtual int
    current()const;
private:
    //map field name to index
    std::map<wxString, unsigned>
    field_indices_;

    //field types
    std::vector<int>
    field_types_;

    //a single value with type info
    struct
    value_info_{
        wxString value;
        bool null;
    };

    //locally stored data
    std::vector<std::vector<value_info_> >
    data_;

    //copy other object
    inline void
    make_copy_(const Cached_result &src);

    //find field_info by index
    inline std::map<wxString, unsigned>::iterator
    find_field_idx_(int idx);

    //check if index is in range
    inline bool
    check_idx_(int idx);

    //check if data exists
    inline bool
    check_data_(int idx);

    //check if data exists
    inline int
    check_data_(const wxString &field);

    //currently active row
    unsigned
    current_;
};

}

#endif
